"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformPromiseConstructorCall = exports.createPromiseIdentifier = exports.isPromiseClass = void 0;
const lua = require("../../LuaAST");
const diagnostics_1 = require("../utils/diagnostics");
const lualib_1 = require("../utils/lualib");
const typescript_1 = require("../utils/typescript");
const call_1 = require("../visitors/call");
function isPromiseClass(context, node) {
    const type = context.checker.getTypeAtLocation(node);
    return typescript_1.isStandardLibraryType(context, type, undefined) && node.text === "Promise";
}
exports.isPromiseClass = isPromiseClass;
function createPromiseIdentifier(original) {
    return lua.createIdentifier("__TS__Promise", original);
}
exports.createPromiseIdentifier = createPromiseIdentifier;
function transformPromiseConstructorCall(context, node) {
    const expression = node.expression;
    const signature = context.checker.getResolvedSignature(node);
    const params = call_1.transformArguments(context, node.arguments, signature);
    const expressionName = expression.name.text;
    switch (expressionName) {
        case "all":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.PromiseAll, node, ...params);
        case "allSettled":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.PromiseAllSettled, node, ...params);
        case "any":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.PromiseAny, node, ...params);
        case "race":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.PromiseRace, node, ...params);
        case "resolve":
            lualib_1.importLuaLibFeature(context, lualib_1.LuaLibFeature.Promise);
            return lua.createCallExpression(createStaticPromiseFunctionAccessor("resolve", expression), params, node);
        case "reject":
            lualib_1.importLuaLibFeature(context, lualib_1.LuaLibFeature.Promise);
            return lua.createCallExpression(createStaticPromiseFunctionAccessor("reject", expression), params, node);
        default:
            context.diagnostics.push(diagnostics_1.unsupportedProperty(expression.name, "Promise", expressionName));
    }
}
exports.transformPromiseConstructorCall = transformPromiseConstructorCall;
function createStaticPromiseFunctionAccessor(functionName, node) {
    return lua.createTableIndexExpression(lua.createIdentifier("__TS__Promise"), lua.createStringLiteral(functionName), node);
}
//# sourceMappingURL=promise.js.map